import "package:cloud_firestore/cloud_firestore.dart";
import "package:feedback_repository_interface/feedback_repository_interface.dart";
import "package:firebase_auth/firebase_auth.dart";
import "package:shared_preferences/shared_preferences.dart";

///
class FirebaseFeedbackRepository implements FeedbackRepositoryInterface {
  ///
  FirebaseFeedbackRepository(
    FirebaseFirestore? firestore,
    FirebaseAuth? auth,
  )   : firestore = firestore ?? FirebaseFirestore.instance,
        auth = auth ?? FirebaseAuth.instance;

  ///
  final FirebaseFirestore firestore;

  ///
  final FirebaseAuth auth;

  @override
  Future<void> sendFeedback({required FeedbackReport feedback}) async {
    var user = auth.currentUser;

    feedback = feedback.copyWith(
      userId: user!.uid,
    );

    await firestore
        .collection("feedback")
        .withConverter(
          fromFirestore: (snapshot, _) => Report.fromJson(snapshot.data()!),
          toFirestore: (model, _) => model.toJson(),
        )
        .add(feedback);
  }

  @override
  Future<void> sendReport({required IssueReport report}) async {
    var user = auth.currentUser;

    report = report.copyWith(
      userId: user!.uid,
    );

    await firestore
        .collection("issues")
        .withConverter(
          fromFirestore: (snapshot, _) => Report.fromJson(snapshot.data()!),
          toFirestore: (model, _) => model.toJson(),
        )
        .add(report);
  }

  @override
  Future<bool> shouldShowFeedbackPopup({required int showCount}) async {
    var prefs = await SharedPreferences.getInstance();
    var viewCount = prefs.getInt("feedbackViewCount") ?? 0;
    if (viewCount >= showCount) {
      await prefs.setInt("feedbackViewCount", 0);
      return true;
    }
    return false;
  }

  @override
  Future<void> addViewCount() async {
    var prefs = await SharedPreferences.getInstance();
    var viewCount = prefs.getInt("feedbackViewCount") ?? 0;
    viewCount++;
    await prefs.setInt("feedbackViewCount", viewCount);
  }
}
